package io.xjar;

import java.util.Iterator;
import java.io.Writer;
import java.io.OutputStream;
import java.io.Reader;
import java.io.InputStream;
import java.net.URL;
import java.util.Map;
import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.HashMap;
import java.nio.charset.StandardCharsets;
import io.xjar.key.XKey;
import java.io.File;

public class XGo
{
    private static final String CLRF;
    
    public static void make(final File xJar, final XKey xKey) throws IOException {
        final byte[] md5 = XKit.md5(xJar);
        final byte[] sha1 = XKit.sha1(xJar);
        final byte[] algorithm = xKey.getAlgorithm().getBytes(StandardCharsets.UTF_8);
        final byte[] keysize = String.valueOf(xKey.getKeysize()).getBytes(StandardCharsets.UTF_8);
        final byte[] ivsize = String.valueOf(xKey.getIvsize()).getBytes(StandardCharsets.UTF_8);
        final byte[] password = xKey.getPassword().getBytes(StandardCharsets.UTF_8);
        final Map<String, String> variables = new HashMap<String, String>();
        variables.put("xJar.md5", convert(md5));
        variables.put("xJar.sha1", convert(sha1));
        variables.put("xKey.algorithm", convert(algorithm));
        variables.put("xKey.keysize", convert(keysize));
        variables.put("xKey.ivsize", convert(ivsize));
        variables.put("xKey.password", convert(password));
        final URL url = XGo.class.getClassLoader().getResource("xjar/xjar.go");
        if (url == null) {
            throw new IOException("could not find xjar.go");
        }
        final String dir = xJar.getParent();
        final File src = new File(dir, "xjar.go");
        try (InputStream in = url.openStream();
             Reader reader = new InputStreamReader(in);
             BufferedReader br = new BufferedReader(reader);
             OutputStream out = new FileOutputStream(src);
             Writer writer = new OutputStreamWriter(out);
             BufferedWriter bw = new BufferedWriter(writer)) {
            String line;
            while ((line = br.readLine()) != null) {
                for (final Map.Entry<String, String> variable : variables.entrySet()) {
                    line = line.replace("#{" + variable.getKey() + "}", variable.getValue());
                }
                bw.write(line);
                bw.write(XGo.CLRF);
            }
            bw.flush();
            writer.flush();
            out.flush();
        }
    }
    
    private static String convert(final byte[] bytes) {
        final StringBuilder builder = new StringBuilder();
        for (final byte b : bytes) {
            if (builder.length() > 0) {
                builder.append(", ");
            }
            builder.append(b & 0xFF);
        }
        return builder.toString();
    }
    
    static {
        CLRF = System.getProperty("line.separator");
    }
}
